@@ -4,7 +4,7 @@ import stretch_body.hello_utils as hello_utils
 import time
 import logging, logging.config
 import threading
-
+import sys
 
 class DeviceTimestamp:
     def __init__(self):
@@ -30,13 +30,17 @@ class Device:
     """
     Generic base class for all custom Stretch hardware
     """
-    def __init__(self, name=''):
+    def __init__(self, name='',req_params=True):
         self.name = name
         self.user_params, self.robot_params = RobotParams.get_params()
         self.params = self.robot_params.get(self.name, {})
         self.logger = logging.getLogger(self.name)
-        self.timestamp = DeviceTimestamp()
 
+        if self.params == {} and req_params:
+                self.logger.warning('Parameters for device %s not found. Check parameter YAML and device name. Exiting...' % self.name)
+                sys.exit(1)
+
+        self.timestamp = DeviceTimestamp()
         self.thread_active = False
         self.thread_rate_hz = 25.0
         self.thread_stats = None
@@ -83,7 +87,7 @@ class Device:
     def pull_status(self):
         pass
 
-    def home(self):
+    def home(self,end_pos,to_positive_stop, measuring=False):
         pass
 
     def step_sentry(self,robot):
@@ -94,9 +98,38 @@ class Device:
         hello_utils.pretty_print_dict("params", self.params)
 
     def write_device_params(self,device_name, params,fleet_dir=None):
-        rp=hello_utils.read_fleet_yaml(self.user_params['factory_params'],fleet_dir=fleet_dir)
-        rp[device_name]=params
-        hello_utils.write_fleet_yaml(self.user_params['factory_params'],rp,fleet_dir=fleet_dir)
+        raise DeprecationWarning('This method has been deprecated since v0.3.0')
+
+    def write_configuration_param_to_YAML(self,param_name,value,fleet_dir=None):
+        """
+        Update the robot configuration YAML with a new value
+        """
+        self._write_param_to_YAML(param_name,value,filename='stretch_configuration_params.yaml',fleet_dir=fleet_dir)
+
+    def write_user_param_to_YAML(self, param_name, value, fleet_dir=None):
+        """
+        Update the robot configuration YAML with a new value
+        """
+        self._write_param_to_YAML(param_name, value, filename='stretch_user_params.yaml', fleet_dir=fleet_dir)
+
+    def _write_param_to_YAML(self,param_name,value,filename,fleet_dir=None):
+        """
+        Update the YAML with a new value
+        The param_name has the form device.key, or for a nested dictionary, device.key1.key2...
+        For example, write_configuration_param_to_YAML('pimu.config.cliff_zero',100) will set this value to 100 in the YAML
+        """
+        cp = hello_utils.read_fleet_yaml(filename, fleet_dir=fleet_dir)
+        param_keys=param_name.split('.')
+        d=cp
+        for param_key in param_keys:
+            if param_key in d:
+                if param_key==param_keys[-1]:
+                    d[param_key] = value
+                else:
+                    d = d[param_key]
+            else:
+                print('Improper param_name in _write_param_to_YAML. Not able to update %s' % param_name)
+        hello_utils.write_fleet_yaml(filename, cp, fleet_dir=fleet_dir)
 
     # ########### Thread interface #############
 
