@@ -5,10 +5,11 @@ import os
 import time
 import logging
 import numpy as np
-
+import sys
 
 def print_stretch_re_use():
-    print("For use with S T R E T C H (TM) RESEARCH EDITION from Hello Robot Inc.\n")
+    print("For use with S T R E T C H (R) RESEARCH EDITION from Hello Robot Inc.")
+    print("---------------------------------------------------------------------\n")
 
 def create_time_string(time_format='%Y%m%d%H%M%S'):
     """Returns current time formatted as `time_format`
@@ -31,6 +32,13 @@ def deg_to_rad(x):
 def rad_to_deg(x):
     return 180.0*x/math.pi
 
+def confirm(question):
+    reply = None
+    while reply not in ("y", "n"):
+        reply = input(question + " (y/n)").lower()
+    return (reply == "y")
+
+
 def get_display():
     return os.environ.get('DISPLAY', None)
 
@@ -43,6 +51,10 @@ def set_fleet_id(id):
 def get_fleet_directory():
     return os.environ['HELLO_FLEET_PATH']+'/'+get_fleet_id()+'/'
 
+def set_fleet_directory(fleet_path,fleet_id):
+    os.environ['HELLO_FLEET_ID'] = fleet_id
+    os.environ['HELLO_FLEET_PATH'] = fleet_path
+
 def get_stretch_directory(sub_directory=''):
     """Returns path to stretch_user dir if HELLO_FLEET_PATH env var exists
 
@@ -76,29 +88,39 @@ def read_fleet_yaml(f,fleet_dir=None):
     try:
         if fleet_dir is None:
             fleet_dir=get_fleet_directory()
+        else:
+            if fleet_dir[-1] != '/':
+                fleet_dir = fleet_dir + '/'
         with open(fleet_dir+f, 'r') as s:
             p = yaml.load(s,Loader=yaml.FullLoader)
             return {} if p is None else p
     except IOError:
         return {}
 
-def write_fleet_yaml(fn,rp,fleet_dir=None):
+def write_fleet_yaml(fn,rp,fleet_dir=None,header=None):
     if fleet_dir is None:
         fleet_dir = get_fleet_directory()
+    if fleet_dir[-1]!='/':
+        fleet_dir+='/'
     with open(fleet_dir+fn, 'w') as yaml_file:
+        if header is not None:
+            yaml_file.write(header)
         yaml.dump(rp, yaml_file, default_flow_style=False)
 
+
 def overwrite_dict(overwritee_dict, overwriter_dict):
     for k in overwriter_dict.keys():
         if k in overwritee_dict:
-            if type(overwritee_dict[k])==type(overwriter_dict[k]):
+            if (type(overwritee_dict[k])==type(overwriter_dict[k])) or (type(overwritee_dict[k])==int and type(overwriter_dict[k])==float) or (type(overwritee_dict[k])==float and type(overwriter_dict[k])==int):
                 if type(overwritee_dict[k])==dict:
                     overwrite_dict(overwritee_dict[k],overwriter_dict[k])
                 else:
                     overwritee_dict[k]=overwriter_dict[k]
             else:
-                print('Overwritting Factory Params with User Params. Type mismatch for key:',k)
-        else: #If key not present, add anyhow (useful for adding new end_of_arm)
+                print('Overwritting Robot Params. Type mismatch for key:',k)
+                print('Overwriter',overwriter_dict[k])
+                print('Overwritee', overwritee_dict[k])
+        else: #If key not present, add anyhow (useful for adding new params)
             overwritee_dict[k] = overwriter_dict[k]
 
 def pretty_print_dict(title, d):
@@ -163,7 +185,6 @@ class LoopStats():
 
     def mark_loop_end(self):
         self.status['num_loops'] += 1
-
         # First two cycles initialize vars / log
         if not self.ts_loop_start:
             return
@@ -240,7 +261,6 @@ class ThreadServiceExit(Exception):
     """
     pass
 
-
 #Signal handler, must be set from main thread
 def thread_service_shutdown(signum, frame):
     print('Caught signal %d' % signum)
@@ -288,19 +308,26 @@ def is_segment_feasible(segment, v_des, a_des, t=0.0, inc=0.1):
 
     Returns
     -------
-    bool
+    success: bool
         whether the segment is feasible
+    max_v: float
+        Maximum velocity of spline
+    max_a: float
+        Maximum acceleration of spline
     """
     v_des = float(v_des)
     a_des = float(a_des)
+    max_v = 0
+    max_a =0
+    success=True
     while t < segment[0]:
-        _, vel_t, acc_t = evaluate_polynomial_at(segment[1:-1], t)
+        x_t, vel_t, acc_t = evaluate_polynomial_at(segment[1:-1], t)
+        max_v=max(max_v,abs(vel_t))
+        max_a = max(max_a, abs(acc_t))
         if abs(vel_t) > v_des or abs(acc_t) > a_des:
-            return False
-
+            success=False
         t = min(segment[0], t + inc)
-
-    return True
+    return success, max_v, max_a
 
 def generate_quintic_polynomial(i, f):
     """Generate quintic polynomial from two points
@@ -419,3 +446,84 @@ def get_pose_diff(pose0, pose1, translation_atol=2e-3, rotation_atol=2e-2):
             return -distance, 0.0
 
     return 0.0, 0.0
+
+def pseudo_N_to_effort_pct(joint_name,contact_thresh_N):
+    import stretch_body.robot_params
+    d = stretch_body.robot_params.RobotParams.get_params()[1] #Get complete param dict
+    motor_name = {'arm':'hello-motor-arm', 'lift': 'hello-motor-lift', 'base':'hello-motor-left-wheel'}[joint_name]
+    i_feedforward = 0 if joint_name =='base' else d[joint_name]['i_feedforward']
+    iMax_name = 'iMax_neg' if contact_thresh_N<0 else 'iMax_pos'
+    contact_A = (contact_thresh_N / d[joint_name]['force_N_per_A'])+i_feedforward
+    return 100*contact_A / abs(d[motor_name]['gains'][iMax_name])
+
+
+def check_deprecated_contact_model_base(joint,method_name, contact_thresh_N,contact_thresh ):
+    """
+    With RE2 we are transitioning entire stretch fleet to use new API (and effort_pct for the contact model)
+    Catch older code that is using the older API and require updating of code
+    """
+
+    #Check if old parameters still found in YAML
+    if ('contact_thresh_max_N' in joint.params) or ('contact_thresh_N' in joint.params):
+        msg="Robot is using out-of-date contact parameters"
+        msg=msg+'Please run tool RE1_migrate_contacts.py before continuing.\n'
+        msg=msg+'For more details, see https://forum.hello-robot.com/t/476 \n'
+        msg = msg + 'In method %s.%s' % (joint.name, method_name)
+        print(msg)
+        joint.logger.warning(msg)
+        sys.exit(1)
+
+    #Check if code is passing in old values
+    if contact_thresh_N is not None:
+        msg='Use of parameter contact_thresh_N is no longer supported\n'
+        msg= msg + 'Update your code to use (contact_thresh)\n'
+        msg = msg +  'For more details, see https://forum.hello-robot.com/t/476\n'
+        msg=msg+'In method %s.%s'%(joint.name,method_name)
+        print(msg)
+        joint.logger.warning(msg)
+        sys.exit(1)
+
+def check_deprecated_contact_model_prismatic_joint(joint,method_name, contact_thresh_pos_N,contact_thresh_neg_N,contact_thresh_pos,contact_thresh_neg ):
+    """
+    With RE2 we are transitioning entire stretch fleet to use new API (and effort_pct for the contact model)
+    Catch older code that is using the older API and require updating of code
+    For code that was, for example:
+        arm.move_to(x_m=0.1, contact_thresh_pos_N=30.0, contact_thresh_neg_N=-30.0)
+    Should now be:
+        arm.move_to(x_m=0.1, contact_thresh_pos=pseudo_N_to_effort_pct(30.0),
+            contact_thresh_neg=pseudo_N_to_effort_pct(-30.0))
+    """
+
+    #Check if old parameters still found in YAML
+    if ('contact_thresh_max_N' in joint.params) or ('contact_thresh_N' in joint.params) or ('homing_force_N' in joint.params):
+        msg="Robot is using out-of-date contact parameters\n"
+        msg=msg+'Please run tool RE1_migrate_contacts.py before continuing.\n'
+        msg=msg+'For more details, see https://forum.hello-robot.com/t/476 \n'
+        msg = msg + 'In method %s.%s' % (joint.name, method_name)
+        print(msg)
+        joint.logger.warning(msg)
+        sys.exit(1)
+
+    #Check if code is passing in old values
+    if contact_thresh_pos_N is not None or contact_thresh_neg_N is not None:
+        msg='Use of parameters contact_thresh_pos_N and contact_thresh_neg_N is no longer supported\n'
+        msg= msg + 'Update your code to use (contact_thresh_pos, contact_thresh_neg)\n'
+        msg = msg +  'For more details, see https://forum.hello-robot.com/t/476\n'
+        msg=msg+'In method %s.%s'%(joint.name,method_name)
+        print(msg)
+        joint.logger.warning(msg)
+        sys.exit(1)
+
+    #Check if code is passing in new values but not yet migrated
+    if contact_thresh_pos is not None or contact_thresh_neg is not None \
+            or (contact_thresh_pos is None and contact_thresh_neg is None):
+        if ('contact_models' not in joint.params) or ('effort_pct' not in joint.params['contact_models']) or\
+                ('contact_thresh_default' not in joint.params['contact_models']['effort_pct']) or\
+                ('contact_thresh_homing' not in joint.params['contact_models']['effort_pct']) :
+            msg='Effort_Pct contact parameters not available\n'
+            msg = msg + 'Please run tool RE1_migrate_contacts.py before continuing.\n'
+            msg = msg + 'For more details, see https://forum.hello-robot.com/t/476 \n'
+            msg=msg+'In method %s.%s'%(joint.name,method_name)
+            print(msg)
+            joint.logger.warning(msg)
+            sys.exit(1)
\ No newline at end of file
