@@ -17,10 +17,10 @@ The PIMU is the power and IMU Arduino board in the base
 
 class IMUBase(Device):
     """
-    API to the Stretch RE1 IMU found in the base
+    API to the Stretch IMU found in the base
     """
     def __init__(self):
-        Device.__init__(self, 'imu')
+        Device.__init__(self, 'imu',req_params=False)
         #pitch; //-180 to 180, rolls over
         #roll; //-90 to  90, rolls over at 180
         #heading; //0-360.0, rolls over
@@ -121,7 +121,7 @@ class IMU(IMUBase):
 # ##################################################################################
 class PimuBase(Device):
     """
-    API to the Stretch RE1 Power and IMU board (Pimu)
+    API to the Stretch Power and IMU board (Pimu)
     """
     RPC_SET_PIMU_CONFIG = 1
     RPC_REPLY_PIMU_CONFIG = 2
@@ -145,6 +145,8 @@ class PimuBase(Device):
     STATE_LOW_VOLTAGE_ALERT = 256
     STATE_OVER_TILT_ALERT = 512
     STATE_HIGH_CURRENT_ALERT = 1024
+    STATE_CHARGER_CONNECTED = 2048
+    STATE_BOOT_DETECTED = 4096
 
     TRIGGER_BOARD_RESET = 1
     TRIGGER_RUNSTOP_RESET = 2
@@ -156,13 +158,14 @@ class PimuBase(Device):
     TRIGGER_IMU_RESET = 128
     TRIGGER_RUNSTOP_ON = 256
     TRIGGER_BEEP = 512
+    TRIGGER_LIGHTBAR_TEST = 1024
 
 
     def __init__(self, event_reset=False):
         Device.__init__(self, 'pimu')
+        self.config = self.params['config']
         self.lock = threading.RLock()
         self.imu = IMU()
-        self.config = self.params['config']
         self._dirty_config = True
         self._dirty_trigger = False
         self.frame_id_last = None
@@ -171,7 +174,7 @@ class PimuBase(Device):
         self.status = {'voltage': 0, 'current': 0, 'temp': 0,'cpu_temp': 0, 'cliff_range':[0,0,0,0], 'frame_id': 0,
                        'timestamp': 0,'at_cliff':[False,False,False,False], 'runstop_event': False, 'bump_event_cnt': 0,
                        'cliff_event': False, 'fan_on': False, 'buzzer_on': False, 'low_voltage_alert':False,'high_current_alert':False,'over_tilt_alert':False,
-                       'imu': self.imu.status,'debug':0,'state':0,'motor_sync_drop':0,
+                       'charger_connected':False, 'boot_detected':False,'imu': self.imu.status,'debug':0,'state':0,'motor_sync_drop':0,
                        'transport': self.transport.status}
         self._trigger=0
         self.ts_last_fan_on=None
@@ -181,7 +184,7 @@ class PimuBase(Device):
             self.runstop_event_reset()
             self.cliff_event_reset()
 
-        self.board_info = {'board_version': None, 'firmware_version': None, 'protocol_version': None}
+        self.board_info = {'board_variant': None, 'firmware_version': None, 'protocol_version': None,'hardware_id':0}
         self.hw_valid = False
         self.ts_last_motor_sync=None
         self.ts_last_motor_sync_warn=None
@@ -190,17 +193,23 @@ class PimuBase(Device):
     # ###########  Device Methods #############
 
     def startup(self, threaded=False):
-        Device.startup(self, threaded=threaded)
-        with self.lock:
-            self.hw_valid = self.transport.startup()
-            if self.hw_valid:
-                # Pull board info
-                self.transport.payload_out[0] = self.RPC_GET_PIMU_BOARD_INFO
-                self.transport.queue_rpc(1, self.rpc_board_info_reply)
-                self.transport.step(exiting=False)
-                return True
+        try:
+            Device.startup(self, threaded=threaded)
+            with self.lock:
+                self.hw_valid = self.transport.startup()
+                if self.hw_valid:
+                    # Pull board info
+                    self.transport.payload_out[0] = self.RPC_GET_PIMU_BOARD_INFO
+                    self.transport.queue_rpc(1, self.rpc_board_info_reply)
+                    self.transport.step(exiting=False)
+                    return True
+                return False
+        except KeyError:
+            self.hw_valid =False
             return False
 
+
+
     def stop(self):
         Device.stop(self)
         if not self.hw_valid:
@@ -211,6 +220,11 @@ class PimuBase(Device):
             self.transport.stop()
             self.hw_valid = False
 
+    def set_config(self,c):
+        with self.lock:
+            self.config=c.copy()
+            self._dirty_config = True
+
     def pull_status(self,exiting=False):
         if not self.hw_valid:
             return
@@ -255,11 +269,14 @@ class PimuBase(Device):
         print('Low Voltage Alert', self.status['low_voltage_alert'])
         print('High Current Alert', self.status['high_current_alert'])
         print('Over Tilt Alert',self.status['over_tilt_alert'])
+        if self.board_info['hardware_id']>0:
+            print('Charger Connected', self.status['charger_connected'])
+            print('Boot Detected', self.status['boot_detected'])
         print('Debug', self.status['debug'])
         print('Timestamp (s)', self.status['timestamp'])
         print('Read error', self.transport.status['read_error'])
         print('Dropped motor sync',self.status['motor_sync_drop'])
-        print('Board version:',self.board_info['board_version'])
+        print('Board variant:',self.board_info['board_variant'])
         print('Firmware version:', self.board_info['firmware_version'])
         self.imu.pretty_print()
 
@@ -289,6 +306,11 @@ class PimuBase(Device):
             self._trigger=self._trigger | self.TRIGGER_BEEP
             self._dirty_trigger=True
 
+    def trigger_lightbar_test(self):
+        with self.lock:
+            self._trigger = self._trigger | self.TRIGGER_LIGHTBAR_TEST
+            self._dirty_trigger = True
+
     # ####################### Utility functions ####################################################
     def imu_reset(self):
         with self.lock:
@@ -299,14 +321,18 @@ class PimuBase(Device):
         #Push out immediately
         if not self.hw_valid:
             return
-
         t = time.time()
-        if self.ts_last_motor_sync is not None and t-self.ts_last_motor_sync<1.0/self.params['max_sync_rate_hz']:
-            self.status['motor_sync_drop'] += 1
-            if self.ts_last_motor_sync_warn is None or t-self.ts_last_motor_sync_warn>5.0:
-                print('Warning: Rate of calls to Pimu:trigger_motor_sync above maximum frequency of %.2f Hz. Motor commands dropped: %d'%(self.params['max_sync_rate_hz'],self.status['motor_sync_drop']))
-                self.ts_last_motor_sync_warn=t
-            return
+        #For RE1.0 robots (hardware_id==0) the runstop and sync line are shared
+        #This limits the maximum rate that the motor sync can be triggered
+        #For RE2.0 robots the sync rate is limited by the min pulse width (~10ms)
+        if self.ts_last_motor_sync is not None:
+            sync_rate = 1 / (t - self.ts_last_motor_sync)
+            if sync_rate>self.params['max_sync_rate_hz']:
+                self.status['motor_sync_drop'] += 1
+                if self.ts_last_motor_sync_warn is None or t-self.ts_last_motor_sync_warn>5.0:
+                    print('Warning: Rate of calls to Pimu:trigger_motor_sync rate of %f above maximum frequency of %.2f Hz. Motor commands dropped: %d'%(sync_rate,self.params['max_sync_rate_hz'],self.status['motor_sync_drop']))
+                    self.ts_last_motor_sync_warn=t
+                return
 
         with self.lock:
             self.transport.payload_out[0] = self.RPC_SET_MOTOR_SYNC
@@ -366,7 +392,10 @@ class PimuBase(Device):
     def unpack_board_info(self,s):
         with self.lock:
             sidx=0
-            self.board_info['board_version'] = unpack_string_t(s[sidx:], 20)
+            self.board_info['board_variant'] = unpack_string_t(s[sidx:], 20)
+            self.board_info['hardware_id'] = 0
+            if len(self.board_info['board_variant'])==6: #New format of Pimu.x Older format of Pimu.BoardName.Vx' If older format,default to 0
+                self.board_info['hardware_id']=int(self.board_info['board_variant'][-1])
             sidx += 20
             self.board_info['firmware_version'] = unpack_string_t(s[sidx:], 20)
             self.board_info['protocol_version'] = self.board_info['firmware_version'][self.board_info['firmware_version'].rfind('p'):]
@@ -507,6 +536,8 @@ class Pimu_Protocol_P0(PimuBase):
             self.status['low_voltage_alert'] = (self.status['state'] & self.STATE_LOW_VOLTAGE_ALERT) != 0
             self.status['high_current_alert'] = (self.status['state'] & self.STATE_HIGH_CURRENT_ALERT) != 0
             self.status['over_tilt_alert'] = (self.status['state'] & self.STATE_OVER_TILT_ALERT) != 0
+            self.status['charger_connected'] = (self.status['state'] & self.STATE_CHARGER_CONNECTED) != 0
+            self.status['boot_detected'] = (self.status['state'] & self.STATE_BOOT_DETECTED) != 0
             self.status['timestamp'] = self.timestamp.set(unpack_uint32_t(s[sidx:])); sidx += 4
             self.status['bump_event_cnt'] = unpack_uint16_t(s[sidx:]);sidx += 2
             self.status['debug'] = unpack_float_t(s[sidx:]); sidx += 4
@@ -542,7 +573,9 @@ class Pimu_Protocol_P1(PimuBase):
             self.status['low_voltage_alert'] = (self.status['state'] & self.STATE_LOW_VOLTAGE_ALERT) != 0
             self.status['high_current_alert'] = (self.status['state'] & self.STATE_HIGH_CURRENT_ALERT) != 0
             self.status['over_tilt_alert'] = (self.status['state'] & self.STATE_OVER_TILT_ALERT) != 0
-
+            if self.board_info['hardware_id']>0:
+                self.status['charger_connected'] = (self.status['state'] & self.STATE_CHARGER_CONNECTED) != 0
+                self.status['boot_detected'] = (self.status['state'] & self.STATE_BOOT_DETECTED) != 0
             self.status['timestamp'] = self.timestamp.set(unpack_uint64_t(s[sidx:])); sidx += 8
             self.imu.status['timestamp'] = self.status['timestamp']
             self.status['bump_event_cnt'] = unpack_uint16_t(s[sidx:]);sidx += 2
@@ -553,7 +586,7 @@ class Pimu_Protocol_P1(PimuBase):
 # ######################## PIMU #################################
 class Pimu(PimuBase):
     """
-    API to the Stretch RE1 Power and IMU board (Pimu)
+    API to the Stretch Power and IMU board (Pimu)
     """
     def __init__(self, event_reset=False):
         PimuBase.__init__(self, event_reset)
