@@ -4,7 +4,7 @@ from stretch_body.dynamixel_hello_XL430 import DynamixelHelloXL430
 
 class StretchGripper(DynamixelHelloXL430):
     """
-    API to the Stretch RE1 Gripper
+    API to the Stretch Gripper
     The StretchGripper motion is non-linear w.r.t to motor motion due to its design
     As such, the position of the gripper is represented at as unit-less value, 'pct'
     The Pct ranges from approximately -100 (fully closed) to approximately +50 (fully open)
@@ -18,6 +18,9 @@ class StretchGripper(DynamixelHelloXL430):
                       'open': self.pct_max_open,
                       'close': -100}
 
+    def startup(self, threaded=True):
+        return DynamixelHelloXL430.startup(self, threaded=threaded)
+
     def home(self,move_to_zero=True):
         DynamixelHelloXL430.home(self,single_stop=True,move_to_zero=move_to_zero,delay_at_stop=3.0)
 
