@@ -10,7 +10,7 @@ import textwrap
 
 class WaccBase(Device):
     """
-    API to the Stretch RE1 wrist+accelerometer (Wacc) board
+    API to the Stretch wrist+accelerometer (Wacc) board
     The Wacc has:
     -- 3-axis accelerometer reported as Ax,Ay,and Az
     -- Two digital inputs D0, D1
@@ -37,7 +37,6 @@ class WaccBase(Device):
         self.ext_status_cb=ext_status_cb
         self.ext_command_cb=ext_command_cb
         self.lock=threading.RLock()
-        self.config = self.params['config']
         self._dirty_config = True #Force push down
         self._dirty_command = False
         self._command = {'d2':0,'d3':0, 'trigger':0}
@@ -45,23 +44,30 @@ class WaccBase(Device):
         self.status = { 'ax':0,'ay':0,'az':0,'a0':0,'d0':0,'d1':0, 'd2':0,'d3':0,'single_tap_count': 0, 'state':0, 'debug':0,
                        'timestamp': 0,
                        'transport': self.transport.status}
-        self.board_info = {'board_version': None, 'firmware_version': None, 'protocol_version': None}
+        self.board_info = {'board_variant': None, 'firmware_version': None, 'protocol_version': None,'hardware_id':0}
         self.hw_valid = False
 
     # ###########  Device Methods #############
 
     def startup(self, threaded=False):
-        Device.startup(self, threaded=threaded)
-        with self.lock:
-            self.hw_valid = self.transport.startup()
-            if self.hw_valid:
-                # Pull board info
-                self.transport.payload_out[0] = self.RPC_GET_WACC_BOARD_INFO
-                self.transport.queue_rpc(1, self.rpc_board_info_reply)
-                self.transport.step(exiting=False)
-                return True
+        try:
+            self.config = self.params['config']
+            Device.startup(self, threaded=threaded)
+            with self.lock:
+                self.hw_valid = self.transport.startup()
+                if self.hw_valid:
+                    # Pull board info
+                    self.transport.payload_out[0] = self.RPC_GET_WACC_BOARD_INFO
+                    self.transport.queue_rpc(1, self.rpc_board_info_reply)
+                    self.transport.step(exiting=False)
+                    return True
+                return False
+        except KeyError:
+            self.hw_valid =False
             return False
 
+
+
     def stop(self):
         Device.stop(self)
         if not self.hw_valid:
@@ -129,7 +135,7 @@ class WaccBase(Device):
         print('State ', self.status['state'])
         print('Debug',self.status['debug'])
         print('Timestamp (s)', self.status['timestamp'])
-        print('Board version:', self.board_info['board_version'])
+        print('Board variant:', self.board_info['board_variant'])
         print('Firmware version:', self.board_info['firmware_version'])
 
     # ####################### Utility functions ####################################################
@@ -143,7 +149,10 @@ class WaccBase(Device):
     def unpack_board_info(self,s):
         with self.lock:
             sidx=0
-            self.board_info['board_version'] = unpack_string_t(s[sidx:], 20)
+            self.board_info['board_variant'] = unpack_string_t(s[sidx:], 20)
+            self.board_info['hardware_id'] = 0
+            if len(self.board_info['board_variant'])==6: #New format of Wacc.x  Older format of Wacc.BoardName.Vx' If older format,default to 0
+                self.board_info['hardware_id']=int(self.board_info['board_variant'][-1])
             sidx += 20
             self.board_info['firmware_version'] = unpack_string_t(s[sidx:], 20)
             self.board_info['protocol_version'] = self.board_info['firmware_version'][self.board_info['firmware_version'].rfind('p'):]
@@ -253,7 +262,7 @@ class Wacc_Protocol_P1(WaccBase):
 # ######################## PIMU #################################
 class Wacc(WaccBase):
     """
-    API to the Stretch RE1 Power and IMU board (Pimu)
+    API to the Stretch Wrist Accelerometer (Wacc) Board
     """
     def __init__(self):
         WaccBase.__init__(self)
