@@ -13,7 +13,7 @@ class TestHelloUtils(unittest.TestCase):
         with warnings.catch_warnings(record=True) as w:
             warnings.simplefilter("always")
             # read yaml, generating a ResourceWarning if the file is not released
-            yaml = stretch_body.hello_utils.read_fleet_yaml('stretch_re1_user_params.yaml')
+            yaml = stretch_body.hello_utils.read_fleet_yaml('stretch_user_params.yaml')
             self.assertTrue(len(w) == 0)
 
     def test_reading_invalid_yaml(self):
